// Copyright 2015 - 2022, GIBIS-UNIFESP and the WiRedPanda contributors
// SPDX-License-Identifier: GPL-3.0-or-later

#pragma once

#include <QObject>

class TestIcons : public QObject
{
    Q_OBJECT

private slots:
    void testIcons();
};
